FLIGHTMAPTIMES_FADESTEP = 0.05;

-- Format seconds into m:ss
local function lFMT_NiceTime(secs)
    return string.format("%d:%02d", secs / 60, math.mod(secs, 60));
end

-- TODO: this should not be replicated here
local function lGetFlightMap()
    local _, faction = UnitFactionGroup("player");
    if not faction then return {}; end
    if not FlightMap[faction] then FlightMap[faction] = {}; end
    return FlightMap[faction];
end

-- TODO: this probably shouldn't be, either
local function lGetContinent()
    -- TODO: should I bother saving/restoring map?
    SetMapToCurrentZone();
    return GetCurrentMapContinent();
end

-- Return a constructed node name from continent number and node ID
local function lGetNodeName(cont, id)
    local x, y = TaxiNodePosition(id);
    return string.format("%d:%d:%d", cont, x * 1000, y * 1000);
end

-- Hook the TakeTaxiNode() call to determine where the player is
-- flying to.
local lFMT_OldTakeTaxiNode;
function FlightMapTimes_TakeTaxiNode(id)
    -- Get ourselves airborne
    lFMT_OldTakeTaxiNode(id);

    -- Establish where we are and where we're going
    local thisNode, thatNode = nil, id;
    for index = 1, NumTaxiNodes(), 1 do
        local tType = TaxiNodeGetType(index)
        if (tType == "CURRENT") then
            thisNode = index;
            break;
        end
    end

    -- If both thisNode and thatNode exist...
    if (FlightMap.Opts.useTimer and thisNode and thatNode) then
        -- Dig out the flight time data
        local map = lGetFlightMap();

        -- Get the current continent number
        local cont = lGetContinent();

        -- Construct node names
        local source = lGetNodeName(cont, thisNode);
        local dest = lGetNodeName(cont, thatNode);

        -- Check if this flight path has a known duration
        if map[source] and map[source].Flights[dest] then
            -- Time exists, sop set it up
            FlightMapTimesFrame.startTime = GetTime();
            FlightMapTimesFrame.endTime = FlightMapTimesFrame.startTime
                                        + map[source].Flights[dest];
            FlightMapTimesStatusBar:SetMinMaxValues(
                    FlightMapTimesFrame.startTime,
                    FlightMapTimesFrame.endTime);
            FlightMapTimesStatusBar:SetValue(
                    FlightMapTimesFrame.startTime);
            FlightMapTimesStatusBar:SetStatusBarColor(1.0, 0.7, 0.0);
            FlightMapTimesSpark:Show();
        else
            -- endTime of nil means we're timing this flight
            FlightMapTimesFrame.startTime = GetTime();
            FlightMapTimesFrame.endTime = nil;
            FlightMapTimesStatusBar:SetMinMaxValues(0, 100);
            FlightMapTimesStatusBar:SetValue(100);
            FlightMapTimesStatusBar:SetStatusBarColor(0.0, 0.0, 1.0);
            FlightMapTimesSpark:Hide();
        end
        -- Always keep track of the source and destination nodes,
        -- and the unqualified destination label
        FlightMapTimesFrame.sourceNode = source;
        FlightMapTimesFrame.destNode = dest;
        FlightMapTimesFrame.started = false;

        -- Dig out a shorter and easier on the eye name for the
        -- destination
        local nodeName = TaxiNodeName(thatNode);
        local comma = string.find(nodeName, ", ");
        if comma then
            nodeName = string.sub(nodeName, 1, comma - 1);
        end
        FlightMapTimesFrame.endPoint = nodeName;

        -- Prepare to fade in
        FlightMapTimesFrame:SetAlpha(0);
        FlightMapTimesFrame:Show();
    else
        FlightMapTimesFrame:Hide();
    end
end

-- Hook TaxiNodeOnButtonEnter() to show flight times
-- Unfortunately, this needs to be a replace, not an override
-- because I want the time between the money and the label...
local lFMT_OldTaxiNodeOnButtonEnter;
function FlightMapTimes_TaxiNodeOnButtonEnter(button)
    local index = this:GetID();
    local type = TaxiNodeGetType(index);

    -- from original function
    GameTooltip:SetOwner(button, "ANCHOR_RIGHT");
    GameTooltip:AddLine(TaxiNodeName(index), "", 1.0, 1.0, 1.0);

    if type == "REACHABLE" then
        -- Get the current continent number
        local cont = lGetContinent();
        local map = lGetFlightMap();

        -- Construct node names
        local dest = lGetNodeName(cont, index);
        local source;

        -- Find the source node
        for i = 1, NumTaxiNodes(), 1 do
            if TaxiNodeGetType(i) == "CURRENT" then
                source = lGetNodeName(cont, i);
                break;
            end
        end

        if source and map[source] and map[source].Flights
        and map[source].Flights[dest] and map[source].Flights[dest] ~= 0 then
            local fmt = lFMT_NiceTime(map[source].Flights[dest]);
            GameTooltip:AddLine("Flight time: " .. fmt, "", 0.9, 0.9, 0.9);
        end

        SetTooltipMoney(GameTooltip, TaxiNodeCost(index));
    elseif type == "CURRENT" then
        GameTooltip:AddLine(TEXT(TAXINODEYOUAREHERE), "", 0.5, 1.0, 0.5);
    end

    GameTooltip:Show();
end

function FlightMapTimes_OnLoad()
    if (Sea) then
        lFMT_OldTakeTaxiNode = function() end;
        lFMT_OldTaxiNodeOnButtonEnter = function() end;
        Sea.util.hook("TakeTaxiNode", "FlightMapTimes_TakeTaxiNode", "after");
        Sea.util.hook("TaxiNodeOnButtonEnter",
                "FlightMapTimes_TaxiNodeOnButtonEnter", "replace");
    else
        lFMT_OldTakeTaxiNode = TakeTaxiNode;
        lFMT_OldTaxiNodeOnButtonEnter = TaxiNodeOnButtonEnter;
        TakeTaxiNode = FlightMapTimes_TakeTaxiNode;
        TaxiNodeOnButtonEnter = FlightMapTimes_TaxiNodeOnButtonEnter;
    end
    this:RegisterForDrag("LeftButton");
end

function FlightMapTimes_OnUpdate()
    -- Gotta wait until we mount up...
    if (not this.started) then
        if UnitOnTaxi("player") then this.started = true; end
        return;
    end

    -- Make sure the user is still airborne
    if (not UnitOnTaxi("player")) then
        -- Fade the bar out
        local alpha = this:GetAlpha() - FLIGHTMAPTIMES_FADESTEP;
        if (alpha > 0) then
            this:SetAlpha(alpha);
        else
            -- Hide up, reset alpha
            this:Hide();
            this:SetAlpha(1.0);
            -- If no time existed, or it was too long, save the recorded time
            if (not this.endTime or (GetTime() < this.endTime)) then
                local length = GetTime() - this.startTime;
                local map = lGetFlightMap();
                if map[this.sourceNode] then
                    map[this.sourceNode].Flights[this.destNode] = length;
                end
            end
        end
    else
        local label = this.endPoint .. ": ";
        local now = GetTime();
        -- If the time was too short, wipe it out and save a new one!
        if (this.endTime and this.endTime < now) then
            this.endTime = nil;
            FlightMapTimesStatusBar:SetMinMaxValues(0, 100);
            FlightMapTimesStatusBar:SetValue(100);
            FlightMapTimesStatusBar:SetStatusBarColor(0.0, 0.0, 1.0);
            FlightMapTimesSpark:Hide();
        end

        -- Update the spark, status bar and label
        if (this.endTime) then
            local remains = this.endTime - now;
            label = label .. lFMT_NiceTime(remains);
            local sparkPos = ((now - this.startTime)
                            / (this.endTime - this.startTime)) * 195;
            FlightMapTimesSpark:SetPoint("CENTER",
                    "FlightMapTimesStatusBar", "LEFT", sparkPos, 0);
            FlightMapTimesStatusBar:SetValue(now);
        else
            label = label .. FLIGHTMAP_TIMING;
        end
        FlightMapTimesText:SetText(label);

        -- If alpha is below one, fade-in is active
        local alpha = this:GetAlpha();
        if (alpha < 1) then
            alpha = alpha + FLIGHTMAPTIMES_FADESTEP * 4;
            if (alpha > 1) then alpha = 1; end
            this:SetAlpha(alpha);
        end
    end
end

-- Movable window
function FlightMapTimes_OnDragStart()
    FlightMapTimesFrame:StartMoving();
end

function FlightMapTimes_OnDragStop()
    FlightMapTimesFrame:StopMovingOrSizing();
end
